# Elisa - Home multimedia server
# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.
#
"""
This is a basic Elisa client which print infos about current playing
media. It can also send actions to the player. Actions are strings
like:

- play
- pause
- toggle_play_pause
- next
- prev
- seek_forward
- seek_backward
- increment_playback_speed
- decrement_playback_speed
- stop

If you supply no action the client will just display informations
about the playing media.
"""

from twisted.spread import pb
from twisted.internet import reactor
import sys, getopt

def got_infos(infos):
    print infos

def print_item_status(player):
    dfr = player.callRemote("get_status_informations")
    dfr.addCallback(got_infos)
    return dfr

def action_called(result, action, factory):
    if result is not None:
        print 'Action %r result : %s' % (action, result)
    return factory.getRootObject()

def send_action(factory, player, action):
    dfr = player.callRemote(action)
    dfr.addCallback(action_called, action, factory)
    return dfr

def main():
    args = sys.argv[1:]
    action = ""
    host = "localhost"
    
    try:
        optlist,args = getopt.getopt(args,"",["host=", "action="])
    except getopt.GetoptError, error:
        print error
        return 0
    else:
        for o,a in optlist:
            if o == '--host':
                host = a
            elif o == '--action':
                action = a
        
    factory = pb.PBClientFactory()
    reactor.connectTCP(host, 8789, factory)
    d = factory.getRootObject()
    d.addCallback(lambda object: object.callRemote("get_player"))
    if action:
        d.addCallback(lambda player: send_action(factory, player, action))
        d.addCallback(lambda object: object.callRemote("get_player"))

    d.addCallback(lambda player: print_item_status(player))
    d.addCallback(lambda _: reactor.stop())
    reactor.run()
    return 0

if __name__ == '__main__':
    sys.exit(main())
