# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

from elisa.core.base_plugins import QueuePlugin
from elisa.core.interfaces import IQueue
from elisa.core import media_uri

class DefaultQueue(QueuePlugin):

    __implements__ = IQueue

    name = "default_queue"

    provides = "player:queue"

    def __init__(self):
        QueuePlugin.__init__(self)
        self.history = []
        self.current_uri = None
        self.current_short_name = None

    def get_current_item(self):
        """ Return the current uri of the playing queue as a tuple
        (uri, short_name)
        """
        return (self.current_uri, self.current_short_name)

    def set_current_item(self, uri, short_name):
        """ Set the current uri of the playing queue as a tuple (uri,
        short_name)
        """
        old = self.get_current_item()
        if old:
            self.history.append(old)

        self.current_uri = uri
        self.current_short_name = short_name
        
    def previous_item(self):
        """ Select the previous uri of the queue and return it as a
        tuple (uri, short_name)
        """
        try:
            uri, short_name = self.history[-1]
        except IndexError:
            uri = None
            short_name = None
        else:
            self.history = self.history[:-1]
        #self.set_current_item(uri)
        return uri, short_name

    def next_item(self, head_uri=None, current=None, at_user_request=False):
        """ Select the next uri of the queue and return it as a tuple
        (uri, short_name)
        """
        next_location = None
        short_name = None
        if not current:
            current, short_name = self.get_current_item()

        self.logger.info("Head URI: %s . Current URI: %s" % (head_uri, current))

        if head_uri:
            if not current:
                current = head_uri

            data_plugin = self.get_data_plugin(head_uri)
            if data_plugin:
                location = data_plugin.next_location(current)
                if location:
                    next_location, short_name = location
                while next_location:
                    uri = media_uri.Uri(location=next_location)
                    # FIXME: is_valid_media SHOULD be rewritten to use movies/music/... item_filter
                    if uri.is_valid_media():
                        break
                    else:
                        next_location, short_name = data_plugin.next_location(next_location)

        return next_location, short_name
