# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

import sys

import pygst
pygst.require('0.10')
import gst

from twisted.spread import pb
from elisa.core import media_uri, plugin, interfaces, common
import os

SUBTITLE_FORMATS=("asc", "txt", "sub", "srt", "smi", "ssa")

class Playable(pb.Referenceable):

    """

    """

    data_access = plugin.ExtensionPoint(interfaces.IDataAccess)


    def __init__(self, uri, name="playable", contains_audio=True,
                 contains_video=True, length=-1):
        self._uri = uri
        self._name = name
        self._contains_audio = contains_audio
        self._contains_video = contains_video
        self.set_status(0)
        self.set_length(length)
        self.set_volume(20)
        self.save_volume()

    def __repr__(self):
        return "%s (%s)" % (self.get_name(), self.get_uri())

    def get_id(self):
        return id(self)

    def get_volume(self):
        return self._volume

    def set_volume(self, volume):
        self._volume = volume

    def save_volume(self):
        volume = self.get_volume()
        if volume:
            self._saved_volume = volume

    def get_saved_volume(self):
        return self._saved_volume

    def get_name(self):
        return self._name

    def get_uri(self):
        return self._uri

    remote_get_uri = get_uri

    def get_subtitle_uri(self):
        uri = media_uri.Uri(location=self._uri)
        path = uri.get_parts()['path']
        basename, dummy = os.path.splitext(path)
        data_plugin = self.data_access.get("uri:%s" % uri.scheme)
        subtitle_uri = None
        if data_plugin and uri.scheme != 'http':
            for ext in SUBTITLE_FORMATS:
                sub_uri = uri.copy()
                sub_uri.path = '%s.%s' % (basename, ext)
                sub_uri = str(sub_uri)
                if not data_plugin.is_dir(sub_uri):
                    handle = data_plugin.open(sub_uri)
                    if handle:
                        data_plugin.close(handle)
                        subtitle_uri = sub_uri
                        break
        return subtitle_uri
                
    def contains_audio(self):
        return self._contains_audio

    def contains_video(self):
        return self._contains_video

    def set_status(self, status):
        self._status = status

    def get_status(self):
        return self._status

    def get_length(self):
        return self._length

    def set_length(self, length):
        self._length = length

    def get_status_string(self):
        position = self.get_status()
        duration = self.get_length()
        status = ""
        if position != gst.CLOCK_TIME_NONE:
            position = gst.TIME_ARGS(position).split('.')[0]
            if duration > 0:
                duration = gst.TIME_ARGS(duration).split('.')[0]
            else:
                duration = 'CLOCK_TIME_NONE'
            if duration == 'CLOCK_TIME_NONE':
                status = position
            else:
                status = "%s / %s" % (position, duration)
        return status

    remote_get_status_string = get_status_string

    def get_status_informations(self):
        infos = ""
        if self.contains_audio() and not self.contains_video():
            media_mgr = common.get_application().get_media_manager()
            if media_mgr.enabled():
                media = media_mgr.get_media_for_uri(self.get_uri())
                if media:
                    if media.artist != 'unknown artist':
                        infos += "Artist  : %s\n" % media.artist
                    if media.album != 'unknown album':
                        infos += "Album   : %s\n" % media.album
                    infos +=     "Song    : %s\n" % media.song
                    infos +=     "Rating  : %s\n" % media.rating

        infos += """\
URI     : %s
Status  : %s
Volume  : %s%%""" % (self.get_uri(), self.get_status_string(),
                     self.get_volume())
        return infos
