# -*- coding: utf-8 -*-
# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php

# Copyright (C) 2006,2007, Elisa team <contact@fluendo.com>

from elisa.core.interfaces import IMediaPlayer
from elisa.core.base_plugins import PlayerPlugin
from elisa.core import menu, common
import elisa.core.utils.misc

class DVDTreePlugin(PlayerPlugin):
    """
    dvd plugin_tree

    TODO:

    - implement IDataAccess
    - menu support
    - title/chapters navigation
    - audio track selection
    - subtitle selection


    """

    __implements__ = IMediaPlayer

    name = "dvd"
    menu_name = _("DVD")
    menu_help_string = _("Watch a DVD")
    
    icons = ["next","previous","dvd_menu"] + PlayerPlugin.icons
    
    folder_action = None
    
    def __init__(self):
        PlayerPlugin.__init__(self)
        self.item_action_menu(self)

    def item_action_menu(self, menu_item):

        dvd_menu = menu.MenuItem(short_name=_("Menu"))
        dvd_menu.set_icon_path(self.dvd_menu_icon)
        dvd_menu.set_action_callback(self.open_menu)

        #menu_item.add_item(dvd_menu)

        PlayerPlugin.item_action_menu(self, menu_item)

        for target in ("rate+","rate-","remove"):
            menu_item.del_item_with_target(target)

    def open_menu(self, menu_item):
        pass

    def play_item(self, menu_item):
        appli = common.get_application()
        menu_item = appli._root_menuitem.get_item_with_target(menu_item.name)

        skin_manager = appli.get_skin_manager()
        uri = 'dvd://'

        player = self.get_player()

        def launch(r):

            common.get_application().load_stream_uri(player, uri)
            player.play()

            self.logger.info("Playing item: %s" % uri)

            PlayerPlugin.play_item(self, menu_item)
            
        
        dfr = player.set_uri(uri)
        if dfr:
            dfr.addCallback(launch)


    def playing(self, menu_item):
        for target in ("previous","next"):
            item = menu_item.get_item_with_target(target)
            if item:
                item.enable()
        PlayerPlugin.playing(self,menu_item)


    def add_wizard_shortcuts(self):
        pass
