# -*- coding: utf-8 -*-
# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php
# Copyright(C) 2006,2007, Elisa team <contact@fluendo.com>

from elisa.core import plugin
from elisa.core.base_plugins import ServerPlugin
from elisa.core import common
from elisa.core import interfaces
from elisa.core.interfaces import IResourceProvider
from elisa.core.signals.application import application_started

from twisted.internet import reactor
from twisted.web import resource, static, server
from twisted.python import util

import os

class RootResource(resource.Resource):
    addSlash = True

    def __init__(self):
        resource.Resource.__init__(self)

    def render(self, request):
        return """
        <html>
        <head>
        <title>Elisa HTTP Server</title>
        </head>
        <body>
        <h1>Empty page</h1>
        </body>
        </html>
        """

    def getChild(self, path, request):
        if (path == ''):
            return self
        else:
            return RootResource()

class SubResource(resource.Resource):
    addSlash = True
    def __init__(self, name):
        resource.Resource.__init__(self)
        self._name = name

    def render(self, request):
        return """
        <html>
        <head>
        <title>Elisa HTTP Server</title>
        </head>
        <body>
        <h1>%s Resource</h1>
        </body>
        </html>
        """ % self._name

    def getChild(self, path, request):
        if (path == ''):
            return self
        else:
            return SubResource(self._name)

class HttpServerPlugin(ServerPlugin):
    __platforms__ = [] #twisted should be platform independent
    provides = ['http']
    singleton = True
    name = 'http_server'
    resource_plugins = plugin.ExtensionPoint(interfaces.IResourceProvider)            
    _port = 8000

    def __init__(self):
        ServerPlugin.__init__(self)
        self._root = resource.Resource() #RootResource()
        self._root.putChild('', RootResource())
        application_started.connect(self._application_started)
        self._twisted_port = reactor.listenTCP(self._port, server.Site(self._root))        

    def get_server_port(self):
        return self._port

    def _application_started(self):
        for k, v in self.resource_plugins.iteritems():
            if k.startswith('http:'):
                self.logger.info("Adding web resource handler for: " + k[5:])
                pos = k.find('/', 5)
                if pos != -1:
                    sub = SubResource(k[5:pos])
                    self._root.putChild(k[5:pos], sub)
                    print k[pos + 1:]
                    if (pos + 1 <= len(k)):
                        sub.putChild(k[pos + 1:], v.get_resource())
                else:
                    self._root.putChild(k[5:], v.get_resource())

