# -*- coding: utf-8 -*-
# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php

# Copyright (C) 2006,2007, Elisa team <contact@fluendo.com>

from elisa.core.interfaces import IDataAccess
from elisa.core import base_plugins, common
from elisa.core import common, media_uri
from elisa.core.utils import misc

class MetaFSPlugin(base_plugins.DataAccessPlugin):

    __implements__ = IDataAccess
    name = 'meta_fs'
    provides = ["uri:meta"]
    singleton = True
    uncacheable_uri_schemes = ['meta']
    
    def __init__(self):
        base_plugins.DataAccessPlugin.__init__(self)
        media_manager = common.get_application().get_media_manager()
        if not media_manager.enabled():
            self.logger.info("MetaFS plugin won't work without the media manager")
            self.unload()
        
    def is_dir(self, uri):
        return uri[:7] == 'meta://'

    def open(self, uri):
        """
        Open an uri and return DataAccessFile file
        """
        # FIXME: This is a hack until uri schema for metafs is redesigned
        plugin_manager = common.get_plugin_manager()
        music = plugin_manager.get_plugin_with_name('music')
        cover_fname = music.get_config().get('cover_file')
        cover_path = "/%s" % cover_fname
        if str(uri).endswith(cover_path):
            album_uri = str(uri).split(cover_path)[0]
            cover_uri = self.get_cover_for_uri(album_uri)
            if cover_uri:
                data_access_plugin = self.data_access.get("uri:%s" % cover_uri.scheme)
                return data_access_plugin.open(str(cover_uri))


    def create_menu_for_uri(self, uri, for_plugin, short_name=None):
        if uri == 'meta://artist:list':
            short_name = 'By Artist'
        elif uri == 'meta://album:list':
            short_name = 'By Album'
        return base_plugins.DataAccessPlugin.create_menu_for_uri(self, uri,
                                                                 for_plugin,
                                                                 short_name=short_name)

    def get_direct_children(self, uri):
        media_manager = common.get_application().get_media_manager()

        if uri[:7] != 'meta://':
            #raise 'media library: invalid uri', uri
            return {}

        column = 'song'
        artist_name = 'list'
        album_name = 'list'
        order = []
        base_sub_uri = 'meta:/'
        where_clause = []#"format='audio'",]
        children = {}

        for piece in uri[5:].split('/'):
            if piece[0:7] == 'artist:':
                order.append('artist')
                artist_name = piece[7:]
                if artist_name.lower() == 'list':
                    column = 'artist'
                else:
                    base_sub_uri += '/artist:' + artist_name
                    where_clause.append("artist=%r" % artist_name)
            elif piece[0:6] == 'album:':
                order.append('album')
                album_name = piece[6:]
                if album_name.lower() == 'list':
                    column = 'album'
                else:
                    base_sub_uri += '/album:' + album_name
                    where_clause.append("album=%r" % album_name)

        if column == 'song':
            request = 'select uri, song from audio_media, media'
            where_clause.append('media.id=audio_media.media_id')
            first = 'song'
        else:
            request = 'select distinct(' + column + ') from audio_media'
            first = column
            
        if where_clause:
            request += ' where ' + ' and '.join(where_clause)

        index = 0
        rows = self.sql_execute(request)
        for row in rows:
            if column == 'song':
                if not row.uri:
                    print 'no uri', row.id
                    continue
                sub_uri = row.uri
                child_type = 'file'
            else:
                col = getattr(row, column)
                if not col:
                    print 'no value for %s in row %s' % (column, row.id)
                    continue
                if type(col) == unicode:
                    col = col.encode('utf-8')
                sub_uri = base_sub_uri + '/' + column + ':' + col
                child_type = 'directory'

            friendly_name = getattr(row, first)
            if not friendly_name and first == 'song':
                friendly_name = u'un-named'
            friendly_name = misc.fs_encode(friendly_name)

            file_type = 'audio'
            #file_type = row.typ

            if uri == 'meta://artist:list':
                sub_uri += '/album:list'
            if type(sub_uri) == unicode:
                sub_uri = sub_uri.encode('utf-8')
            children[str(sub_uri)] = (child_type, file_type, friendly_name,
                                      index)
            index += 1
            
        return children

    def has_children(self, uri):
        has_children = True
        if uri == 'meta://artist:list':
            request = "select count(*) as c from audio_media"# where format='audio'"
            result = self.sql_execute(request)
            if result:
                has_children = result[0].c > 0
        return has_children

    def get_cover_for_uri(self, uri):
        cover_uri = None
        uri = media_uri.Uri(location=uri)
        if uri.path:
            splitted = uri.path.split(':')
            sub_typ = splitted[0]
            album_name = ':'.join(splitted[1:])
            if sub_typ == "/album" and album_name != "list":
                request = "select cover_uri from audio_media, media where media.id=audio_media.media_id and album=%r and cover_uri!=''"
                request = request % album_name
                result = self.sql_execute(request)
                if result:
                    cover_uri = result[0].cover_uri
                    cover_uri = media_uri.Uri(cover_uri)
        return cover_uri
        
    def close(self, data_access_file):
        """
        Close the DataAccessFile
        """

    def seek(self, data_access_file, offset, whence=0):
        """
        Close the DataAccessFile
        """

    def read(self, data_access_file, size=-1):
        """
        Close the DataAccessFile
        """

    def remove_location(self, uri):
        """
        Delete data at location identified by its uri.
        """

    def next_location(self, from_uri):
        """
        Return the uri just next to `from_uri`.
        """

if __name__ == '__main__':
    # FIXME: this autotest won't work anymore because media manager is now part
    #        of Application
    from elisa.core.plugin_manager import PluginManager
    from elisa.core.media_manager import MediaManager
    from elisa.core.config import Config
    from elisa.core import common
    from twisted.internet import reactor

    conf = Config("local.conf")
    manager = PluginManager(conf)
    common._set_plugin_manager(manager)
    cache = CacheManager(conf)
    common._set_cache_manager(cache)
    manager.load_plugins(False)

    def test():
        f = MetaFSPlugin()
        f.post_init()
        children = f.get_direct_children
        # retourne tout les artistes
        print children('meta://artist:list')
        # retourne tout les albums
        print children('meta://album:list')
        # retourne tous les album de Coldplay
        print children('meta://artist:Coldplay/album:list')
        print children('meta://artist:Coldplay/album:A Rush Of Blood To The Head')
        reactor.stop()
        
    reactor.callLater(0.4,test)
    reactor.run()
