# -*- coding: utf-8 -*-
# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php

# Copyright (C) 2006, Elisa team <contact@fluendo.com>

from elisa.core.interfaces import IMediaPlayer
from elisa.core.base_plugins import PlayerPlugin
from elisa.core import common, media_uri, menu
import elisa.core.utils.misc
from elisa.core.utils.misc import handle_failure
from elisa.core.player.playable import SUBTITLE_FORMATS
from elisa.core.thumbnailers import video_thumbnailer

import os, sys

class MoviesTreePlugin(PlayerPlugin):
    """
    movies plugin_tree

    """

    __implements__ = IMediaPlayer

    name = "movies"
    menu_name = _("Movies")
    menu_help_string = _("Watch movies")
    folder_icon_path = 'folder-movies.png'
    item_icon_path = 'movie.png'

    config_doc = {'locations': 'the media locations',
                  'enable_demo':''}

    default_config = {'locations': ['file://./sample_data/movies/*',]}

    provides = ["video", "application"]

    preferred_formats = ['vnd.dlna.mpeg-tts', 'mpeg','x-ms-wmv', 'ogg',
                         'octet-stream']
    
    def __init__(self):
        if sys.platform == "win32": #Add My Videos to the default config
            from win32com.shell import shell
            clsid_myvideos = 14
            try:
                p = shell.SHGetSpecialFolderPath(0, clsid_myvideos)
                print "My Videos directory is:", p
                if p:
                    p = media_uri.Uri(p)
                    self.default_config['locations'].append(p.get_location())
            except:
                print "Could not retrieve My Videos directory"

        PlayerPlugin.__init__(self)
        config = self.get_config()
        demo = int(self.get_config().get('enable_demo', '0'))

        uris = [ ("http://195.10.10.101:80/rtve/l2n/ondemand/recording/last.ogg",
                  "La 2 noticias last"),
                 ("http://195.10.10.101:80/rtve/l2n/ondemand/384/HIDDEN_3341_481_VIDEO_TVE-1.ogg", "La 2 noticias Ahimsa"),
                 ("http://195.10.10.102:80/rtve/l2n/ondemand/384/HIDDEN_3341_261_VIDEO_Koldo_Azkune.ogg", "La 2 noticias Koldo Azkune"),
                 ("http://195.10.10.101:80/fluendo/demo/moto/low.ogg",
                  "moto low (ogg)"),
                 ("http://195.10.10.101:80/fluendo/demo/moto/high.ogg",
                  "moto high (ogg)"),
                 ]

        if demo:
            item = menu.MenuItem(short_name=_("Flumotion streams"))
            item.set_icon_path("folder-movies-news.png")
            for (uri, label) in uris:
                data = self.get_data_plugin(uri)
                if data:
                    stream = data.create_menu_for_uri(uri, self, label)
                    item.add_item(stream)
            self.add_item(item)

    def play_item(self, menu_item):
        if not self.lock():
            return
        
        uri = menu_item.get_target_path()
        uri = media_uri.Uri(uri)

        player = self.get_player()
        current_playable = player.get_current_item()
        dfr = None

        def launch(r):
            self.logger.info("Playing item: %s" % player.get_playing_uri())
            common.get_application().load_stream_uri(player, uri)
            player.play(start=True)
            player.un_mute()
            PlayerPlugin.play_item(self, menu_item)
            self.unlock()
            
        if current_playable and current_playable.get_uri() == str(uri):
            if player.get_state() == player.PAUSED:
                player.play()
            else:
                dfr = player.set_uri(str(uri))
        else:
            dfr = player.set_uri(str(uri))

        if dfr:
            dfr.addCallback(launch)
            dfr.addErrback(handle_failure)
        else:
            self.unlock()
            
    item_action = play_item

    def folder_action_menu(self, menu_item):
        pass

    folder_action = None
    
    def item_filter(self, uri):
        uri = media_uri.Uri(location=uri)
        data_plugin = self.get_data_plugin(uri)
        is_item = False
        if data_plugin:
            if uri.scheme == 'file':
                # FIXME: this is bad
                # check extension for special cases we want to handle
                _, ext = os.path.splitext(str(uri))
                ext = ext[1:]
                if ext in ['ogg', 'ogm']:
                    is_item = True
                else:
                    is_item = data_plugin.get_file_type(uri) == 'video'
            else:
                is_item = not data_plugin.is_dir(str(uri))
        return is_item

    def get_folder_thumbnail(self, uri, widget_item, size):
        # TODO
        pass

    #from elisa.core.utils import profiling
    #@profiling.profileit(1)
    def get_item_thumbnail(self, uri, widget_item, size):
        #data_access_plugin = self.data_access.get("uri:%s" % uri.scheme)
        #file = data_access_plugin.open(str(uri))
        th_manager = common.get_application().get_thumbnail_manager()
        try:
            thumbnail = th_manager.get_thumbnail(uri, size, video_thumbnailer.generate_thumbnail)
            widget_item.set_background_image(thumbnail)
        except Exception, e:
            self.logger.info("Thumbnail generation failed for %s: %s:" % (uri, str(e)))
