# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

import pkg_resources
from twisted.internet import defer

from elisa.core.utils import image
from elisa.core.plugins.skins.default import widget_menu, status_bar, \
     loading, player_dock
from elisa.core.base_skin import BaseSkin
from elisa.core.interfaces import ISkin

from elisa.core import common, media_uri, log, menu
from elisa.core.signals.menu import *
from elisa.core.signals.misc import state_changed
from elisa.core.signals.player import stopped_signal, playing_signal, error_signal
from elisa.core.plugins import pictures
from elisa.core.widgets import dialog_box
from elisa.core.base_plugins import UserInputPlugin
from pgm.layout import window

import mutex
import random
import time
import textwrap
import gst
import copy

from pgm.timing import controller, modifier, keyframe
from pgm.form import stream, rectangle, label
from pgm.layout import group
from pgm import widget, render
from pgm import log as pgm_log
#pgm_log.set_level(pgm_log.DEBUG)

"""
TODO:

- slideshow => calculate position varation for hide picture border

POSTPONED:

- menu_item "states" 

MISC:

- playlist
- tag_browser
- display left/right arrows when too many items on a level?
  
"""


class ElisaSkin(BaseSkin):
    """
    
    """

    __implements__ = ISkin

    name = 'default_skin'

    pictures_dir = "pictures"

    config_doc = {'move_left_key': 'key to press to go one menu item on the left',
                  'move_right_key': 'key to press to go one menu item on the right',
                  'move_down_key':'key to press to go one menu level down',
                  'move_up_key':'key to press to go one menu level up',
                  'activate_key':'key to press to activate the current menu item',
                  'toggle_menu_key':'key to press to hide/show the menu',
                  'toggle_fullscreen_key':'key to press to set/unset fullscreen mode',
                  'icon_size':'icon size. Currently supported: [128,128]'\
                              'or [256,256] or [512,512]'
                  }
                  
    default_config = {'move_left_key': 'LEFT',
                      'move_right_key': 'RIGHT',
                      'move_down_key': 'DOWN', 'move_up_key': 'UP',
                      'activate_key': 'RETURN', 'toggle_menu_key': 'SPACE',
                      'toggle_fullscreen_key': 'f',
                      'icon_size': [256,256]}

    def __init__(self, win, root_menu_item):
        BaseSkin.__init__(self, win, root_menu_item)
        
        icon_size = self.get_config().get('icon_size')
        ElisaSkin.icon_size = "%sX%s" % tuple(icon_size)
        
        self._fullscreen_uri = None
        self.playing_stream = False
        
        # FIXME: handle "common" pictures correctly
        arrow_image_path = '../arrow.png'
        self._widget_menu = widget_menu.WidgetMenu(self, arrow_image_path)
        self._widget_menu.opacity = 255
        self._window.add(self._widget_menu)
        self._status_bar = status_bar.StatusBar(size=(win.canvas_size[0], 50))
        self._dock = player_dock.PlayerDock(size=(win.canvas_size[0], 156))
        self.show_status_bar()
        self._anim_ctrl = None
        
        state_changed.connect(self._state_changed)

        self.background_path = self.get_picture_path('../background2.png')
        stopped_signal.connect(self._player_stopped)
        playing_signal.connect(self._player_playing)
        error_signal.connect(self._player_error)
        
    def get_menu_widget(self):
        return self._widget_menu
    
    def prepare(self):
        BaseSkin.prepare(self)
        win_size = self.get_window().canvas_size
        self.old_background = rectangle.Rectangle(size=win_size)
        self.old_background.ordering = widget.FAR
        self.old_background.opacity = 0
        self.old_background.tz = -2

        self._grp.add(self.old_background)
        #self.get_window().add(self.old_background)
        self.set_wallpaper(self.background_path)
        self.loading_label = loading.Loading()

        self.get_window().sig_key_release_event.connect(self.keyrelease_cb)

    def _setup_background_widget(self):
        pgmrender = render.get_renderer()

        win_size = self.get_window().canvas_size
        self._grp = group.Group(size=win_size)
        self._grp.ordering = widget.FAR
        self._window.add(self._grp)
        self._background_widget = rectangle.Rectangle(win_size)
        self._background_widget.ordering = widget.FAR
        self._background_widget.opacity = 255
        self._grp.add(self._background_widget)
        self._background_widget.tz = -1


    def keypress_cb(self, key_event):
        key = key_event.keyval
        pgmrender = render.get_renderer()
        action = self.key_actions.get(key)
        
        # so that it does not overlap with events processed in the player
        player = common.get_application().get_player()
        key_actions = player.fs_key_actions.keys()
        if key in key_actions and self.dock_visible():
            action = None

        if action and hasattr(self, action):
            func = getattr(self, action)
            # FIXME: temporary: should be replaced by signals
            # if the callback accepts a parameter, gives it the event
            if func.func_code.co_argcount == 2:
                func(key_event)
            elif func.func_code.co_argcount == 1:
                func()
        
        if not self.menu_visible() and not self.dock_visible() and player.get_playing_uri() \
            and key_event.source == UserInputPlugin.source_types["remote_control"]:
            self.show_dock(show_controls=False)

    def keyrelease_cb(self, key_event):
        pass
    """
        key = key_event.keyval
        pgmrender = render.get_renderer()
        player = common.get_application().get_player()
        key_actions = self.key_actions.keys()
        key_actions.extend(player.key_actions.keys())
        if player.get_playing_uri() and key in key_actions and \
           not self.dock_visible() and \
           key != pgmrender.KEY_RETURN and not self.menu_visible():
            self.show_dock()
    """
    
    def unload(self):
        dfr = BaseSkin.unload(self)
        if self.menu_visible():
            last = self._widget_menu.children[-1]
            for level in self._widget_menu.children:
                # fade out
                if level == last:
                    cb = lambda ctrl: dfr.callback(None)
                else:
                    cb = None
                level.fade(False, cb)
        else:
            dfr.callback(None)
        return dfr

    def resize_cb(self, resize_event):
        self._window.regenerate_widgets()

    def activate(self, event):
        manager = self.get_menu_manager()
        activated = False
        player = common.get_application().get_player()
        if not manager.is_locked():
            activated = manager.activate_current_menu_item()
            
        if not activated and player.get_playing_uri():
            if not self.menu_visible():
                if event.source == UserInputPlugin.source_types["remote_control"]:
                    if not self.dock_visible():
                        self.show_dock(show_controls=False)
                        
                        # if remote is Apple remote, toggle play/pause
                        appli = common.get_application()
                        remote_type = appli.get_config().get_option('lirc_rc', 'plugins.lirc')
                        if remote_type == "apple_remote.lirc":
                            player.toggle_play_pause()
                    else:
                        player.toggle_play_pause()
                        
                else:
                    if not self.dock_visible():
                        self.show_dock()

        return activated

        
    def move_left(self):
        if BaseSkin.move_left(self):
            self._widget_menu.move_left()
    
    def move_right(self): 
        if BaseSkin.move_right(self):
            self._widget_menu.move_right()

    def move_up(self):
        if BaseSkin.move_up(self):
            self._widget_menu.move_up()
       
    def move_down(self):
        if BaseSkin.move_down(self):
            manager = self.get_menu_manager()
            manager.lock()
            self._widget_menu.move_down()

    def show_dock(self, timeout=1, show_controls=True):
        if self._dock not in self._window:
            self._window.add(self._dock)
        if not self.dock_visible():
            self._dock.timeout = timeout
            self._dock.show(animate=False, show_controls=show_controls)

    def dock_visible(self):
        return self._dock.visible
        
    def toggle_dock(self):
        self._dock.toggle(None)
        
    def hide_dock(self):
        self._dock.timeout = 1
        self._dock.hide()
        
    def get_status_bar(self):
        return self._status_bar
                        
    def show_status_bar(self):
        self._window.add(self._status_bar)
        self._status_bar.position = (0,0)
        self._status_bar.visible = True
        self.hide_dock()
        
    def hide_status_bar(self):
        self._window.remove(self._status_bar)
        self._status_bar.visible = False
    
    def _item_removed(self, menu_item):    
        parent = menu_item.get_parent()
        manager = self.get_menu_manager()
        level_widget = manager.get_level_widget_for_menu_item(parent)
        if level_widget:
            item_widget = manager.get_widget_item_for_menu_item(menu_item)
            level_widget.remove_widget_item(item_widget)
        
    def _state_changed(self, obj, prop, new_value):
        """
        TODO:

        - handle skin update of 'icon-label' and 'enabled' properties
        """
        manager = self.get_menu_manager()
        if isinstance(obj, menu.MenuTree):
            if prop == 'shown':
                if new_value == True:
                    self._widget_menu.show_menu()
                else:
                    self._widget_menu.hide_menu()
        elif isinstance(obj, menu.MenuItem):
            widget = manager.get_widget_item_for_menu_item(obj)
            if widget:
                menu_item = obj
                if prop == 'icon-path':
                    picture_path = self.get_picture_path(new_value)
                    img = image.Image(picture_path)
                    widget.set_background_image(img)
                elif prop == 'action-callback':
                    menu_item.set_action_callback(new_value)
                elif prop == 'icon-label':
                    menu_item.set_short_name(new_value)
                    widget.set_caption(new_value)

    def show_menu(self):
        BaseSkin.show_menu(self)
        self.get_menu_widget().show_menu()
        self.show_status_bar()
        self.hide_dock()
        
    def hide_menu(self):
        BaseSkin.hide_menu(self)
        self.get_menu_widget().hide_menu()
        self.hide_status_bar()

    def toggle_menu(self):
        if self.get_menu_widget().toggle_menu():
            player = common.get_application().get_player()
            if player.get_playing_uri():
                if self.dock_visible():
                    self.hide_dock()
            BaseSkin.toggle_menu(self)            
            
    def get_status_bar(self):
        return self._status_bar

    def get_slideshow(self):
        task_manager = common.get_application().get_task_manager()
        slideshows = task_manager.get_tasks_with_type(pictures.Slideshow)
        slideshow = None
        # we can only have one slideshow at a time anyway
        if slideshows:
            slideshow = slideshows[0]
        return slideshow

    def display_stream(self, player, uri):
        """
        """
        slideshow = self.get_slideshow()
        if slideshow:
            slideshow.abort_slideshow()
            
        if self._anim_ctrl:
            self._anim_ctrl.stop()
            self._anim_ctrl = None
            
        self._background_widget.tz = -1
        self._background_widget.ordering = widget.FAR        
        self._background_widget.position = (0, 0)
            
        self.playing_stream = True
        self.set_background_media(player.get_stream())
        self._fullscreen_uri = uri

    def display_image(self, uri, in_slideshow=False):
        player = common.get_application().get_player()

        if self.playing_stream:
            # stop the video player to make background available
            if player.get_current_item().contains_video():
                player.stop()
            else:
                # stop visualization plugin
                player.set_visualization_plugin(stop=True) 

        img = image.Image(uri)
            
        #if in_slideshow:
        #    self.crossfade_pictures(img)
        #else:
        pgmrender = render.get_renderer()
        self._background_widget.aspect_style = pgmrender.LAYOUT_SCALED

        self.playing_stream = False
        self.set_background_media(img)

    def animate_background(self, crossfade_ctrl, prop, start, end):
        
        if self._anim_ctrl:
            self._anim_ctrl.stop()
            self._anim_ctrl = None
            
        timeline = [keyframe.KeyFrame(start, 0), keyframe.KeyFrame(end, 1)]
        mod = modifier.Modifier([self._background_widget], prop, timeline)
        self._anim_ctrl = controller.Controller(begin=0, duration=3500,
                                                resolution=10,
                                                modifiers=[mod,],
                                                repeat_count=0,
                                                end_behavior=controller.END,
                                                interpolation=controller.SINUSOIDAL)
        self._anim_ctrl.start()

    def crossfade_pictures(self, new_img):
        old_img = self._background_widget.media
        old_par = old_img.size[0] / float(old_img.size[1])
        
        # update widget medias
        self._background_widget.media = new_img
        self.old_background.media = old_img

        prop = 'tx'
        z = 90

        win = self.get_window()
        width, height = win.canvas_size
        war = width / float(height)
        
        img_width, img_height = new_img.size
        par = img_width / float(img_height)
        if par < war:
            prop = 'ty'
            z += 290

        stop = random.randint(30, 50)
        end = [stop, -stop][random.randint(0,1)]
        if end < 0:
            start = -end
        else:
            start = 0

        #if old_par < war:
        #    self.old_background.translation = self._background_widget.translation#(0, 0, self._background_widget.tz)
        #else:
        self.old_background.translation = self._background_widget.translation#(0, 0, z-1)
        self._background_widget.translation = (0, 0, z)
        

        if prop == 'tx':
            self._background_widget.tx = self.old_background.tx = start
        else:
            self._background_widget.ty = self.old_background.ty = start

        def animate(ctr):
            self.animate_background(ctr, prop, start, end)

        out_fade = [keyframe.KeyFrame(255, 0), keyframe.KeyFrame(0, 1)]
        mod_out = modifier.Modifier([self.old_background], 'opacity',
                                    out_fade)
        ctrl = controller.Controller(begin=0, duration=1400, resolution=10,
                                     modifiers=[mod_out,],
                                     repeat_count=0,
                                     end_callback=animate,
                                     end_behavior=controller.END,
                                     interpolation=controller.SINUSOIDAL)
        
        ctrl.start()
            
    def rotate_image(self, uri, angle):
        """
        """
        try:
            BaseSkin.rotate_image(self, uri, angle)
        except TypeError, error:
            self.notify(str(error))
        else:
            self.get_menu_widget().rotate_image(uri, angle)

    def get_fullscreen_uri(self):
        return self._fullscreen_uri

    def _player_stopped(self, playable):
        self._fullscreen_uri = None
        self.reset_background()
        self.show_menu()

    def _player_playing(self, playable, first_time):
        if playable.contains_video():
            self.hide_menu()
            self.show_dock()
        else:
            player = common.get_application().get_player()
            if first_time and player.set_visualization_plugin():
                self.set_background_media(player.get_stream())

    def _player_error(self, error):
        message = textwrap.wrap(error.message, 40)
        if error.code == gst.STREAM_ERROR_CODEC_NOT_FOUND:
            message += [_("You can install additional codecs from "),
                        "https://shop.fluendo.com/ "]
        self.notify(message, font_size=25, v_spacing=27)

    def reset_background(self):
        if self._anim_ctrl:
            self._anim_ctrl.stop()
            self._anim_ctrl = None

        # TODO: Seems to do useless stuff, is that true ?
        self._background_widget.translation = (0, 0, 0)
        self._background_widget.tz = -1
        self._background_widget.ordering = widget.FAR
        self._background_widget.position = (0, 0)
        self.set_wallpaper(self.background_path)
        
        self.old_background.ordering = widget.FAR
        self.old_background.opacity = 0
        self.old_background.tz = -2
        
    def start_loading_animation(self, menu_item):
        # FIXME: do this more beautiful
        #name = menu_item.get_short_name()
        #self.loading_label.text = "Loading %s folder" % name
        #self.get_window().add(self.loading_label)
        pass
        
    def stop_loading_animation(self):
        #self.get_window().remove(self.loading_label)
        pass

    def open_popup(self, text, ok_callback, cancel_callback=None, font_size=25,
                   font_name='Sans', v_spacing=50):
        if type(text) in (list, tuple):
            lines = text
        else:
            lines = textwrap.wrap(text, 40)

        b_size = (100, 100)
        bg = image.Image(self.get_picture_path('../popup.png'))
        ok_path = self.get_picture_path('ok.png', icon_size="128X128")
        cancel_path = self.get_picture_path('cancel.png', icon_size="128X128")
        
        box = dialog_box.DialogBox((571, 365), lines, bg, button_spacing=30,
                                   buttons_size=b_size, font_size=font_size,
                                   font_name=font_name,
                                   vertical_spacing=v_spacing,
                                   modal=True, on_close=self._close_popup)

        ok_menu_item = MenuItem(short_name="")
        ok_menu_item.set_icon_path(ok_path)
        ok_menu_item.set_action_callback(ok_callback)
        box.add_button(ok_menu_item, box.close)
        box.set_default(0)
        
        if cancel_callback:
            cancel_menu_item = MenuItem(short_name="")
            cancel_menu_item.set_icon_path(cancel_path)
            cancel_menu_item.set_action_callback(cancel_callback)
            box.add_button(cancel_menu_item, box.close)
            box.set_default(1)
            
        win = self.get_window()
        box.y = win.canvas_size[1]
        win.add(box)
        box.visible = True
        
        box.ordering = widget.NEAR
        box.center(y=False)
        
        def end_cb(ctr):
            win.sig_key_press_event.connect(box.key_pressed)
            
        prop = 'y'
        duration = 500
        timeline = [ keyframe.KeyFrame(win.canvas_size[1], 0),
                     keyframe.KeyFrame((win.canvas_size[1] - box.height) / 2., 1) ]
        mod = modifier.Modifier([box,], prop, timeline)
        self._ctrl2 = controller.Controller(begin=0, duration=duration,
                                            resolution=10,
                                            modifiers=[mod,],
                                            end_behavior=controller.END,
                                            interpolation=controller.SINUSOIDAL,
                                            end_callback=end_cb)

        self._ctrl2.start()

    def _close_popup(self, popup):
        win = self.get_window()

        def end_cb(ctr):
            win.remove(popup)
            
        prop = 'y'
        duration = 500
        timeline = [ keyframe.KeyFrame((win.canvas_size[1] - popup.height) / 2., 0),
                     keyframe.KeyFrame(win.canvas_size[1], 1),]
        mod = modifier.Modifier([popup,], prop, timeline)
        self._ctrl2 = controller.Controller(begin=0, duration=duration,
                                            resolution=10,
                                            modifiers=[mod,],
                                            end_behavior=controller.END,
                                            interpolation=controller.SINUSOIDAL,
                                            end_callback=end_cb)

        self._ctrl2.start()

    def notify(self, message, font_size=25, font_name='Sans', v_spacing=50):
        self.open_popup(message, ok_callback=lambda: None, v_spacing=v_spacing,
                        font_size=font_size, font_name=font_name)
