# -*- coding: utf-8 -*-
# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php

# Copyright (C) 2006,2007, Elisa team <contact@fluendo.com>

from elisa.core.plugin import Plugin
from elisa.core.media_uri import Uri
from elisa.core.interfaces import IResourceProvider
from elisa.core import menu, common
from elisa.core.utils.misc import get_host_address
from elisa.core.signals.application import application_started

from twisted.web import resource, static, server
from twisted.python import util
import os

class UPnPResource(resource.Resource):
    addSlash = True                        

    def __init__(self):
        resource.Resource.__init__(self)
        
    def render(self, request):
        return """
        <html>
        <head>
        <title>Elisa UPnP page</title>
        </head>
        <body>
        <h1>UPnP page!</h1>
        </body>
        </html>
        """

    def getChild(self, path, request):
        if (path == ''):
            return self
        else:
            media_manager = common.get_application().get_media_manager()
            cm = media_manager.get_db_connection().get_media_with_id(int(path))
            uri = Uri(cm.uri)
            f = uri.host + uri.path
            return static.File(f)
            
class UPnPServerPlugin(Plugin):
    """
    Provides UPnP clients access to UPnP resources

    """
    __implements__ = IResourceProvider
    provides = ['http:data/upnp']
    singleton = True

    name = "upnp_server"

    def __init__(self):
        Plugin.__init__(self)
        application_started.connect(self._application_started)

    def _application_started(self):
        application = common.get_application()
        coherence = application.get_coherence()
        if coherence:
            args = {'name': 'Elisa medias',
                    'content_directory': 'localhost', 'version': 2}
            coherence.add_plugin('ElisaMediaStore', **args)
        else:
            self.unload("UPnP MediaServer won't work without Coherence")
            
    def get_resource(self):
        return UPnPResource()
