# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

from elisa.core.menu import MenuTree
from elisa.core import common
from elisa.core import log
import sys

class SkinManager:

    def __init__(self, skin_name, window, root_menuitem):
        self._skin_name = skin_name
        self._window = window
        self._root_menuitem = root_menuitem
        self.logger = log.Logger()
        self._skin = None
        self.load_all_skins()
        self.load_skin()
        
    def load_all_skins(self):
        plugin_manager = common.get_plugin_manager()
        plugin_manager.load_plugins_for_entry_point('elisa.skins',
                                                    check_in_config=False)
        self._skins = plugin_manager.get_plugins().get('elisa.skins',[])
        if len(self._skins) == 0:
            self.logger.info("I can't run without any skin")
            sys.exit(-1)
        self.logger.info("%s skins loaded" % len(self._skins))
        
    def load_skin(self, name=None):
        if not name:
            name = self._skin_name
        for skin in self._skins:
            if skin.name == name:
                self.logger.info("Loading skin %s" % name)
                self._skin = skin(self._window, self._root_menuitem)
                break

    def get_skin(self):
        return self._skin

    def get_skins(self):
        return self._skins
