# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

from pgm.form import label, color
from pgm.layout import group
from pgm import widget
from elisa.core.widgets.action_item import ActionItem
from elisa.core.widgets import rectangle

class Button(group.Group, ActionItem):

    def __init__(self, text, background_color=None, font_name='Sans',
                 font_size=23, icon=None, on_press=None, label_pos=None,
                 size=(50,50)):
        self.bg = rectangle.Rectangle(size=size)
        
        group.Group.__init__(self, size=size)
        ActionItem.__init__(self)
        #self.text = text
        self.font_name = font_name
        self.font_size = font_size
        self.on_press = on_press
        
        # background
        self.bg.ordering = widget.FAR
        self.add(self.bg)

        # icon
        if icon:
            self.bg.media = icon
            self.size = self.bg.size
        else:
            self.bg.paint = background_color
            
        # label
        self._label = label.Label(size=(self.width, 30), text=text,
                                  font_name=self.font_name,
                                  font_size=self.font_size)
        self._label.y_align = 1.        
        self._label.ordering = widget.NEAR
        if label_pos:
            self._label.position = label_pos

        self.add(self._label)        
        if icon and not text:
            self.remove(self._label)

    def icon__get(self):
        return self.bg.media

    def icon__set(self, media):
        self.bg.media = media

    def size__set(self, size):
        group.Group.size__set(self, size)
        self.bg.size = size

    def text__get(self):
        return self._label.text

    def text__set(self, text):
        self._label.text = text
        if self._label not in self:
            self.add(self._label)

    def __repr__(self):
        return "<Button text=%r>" % self.text
