# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

from pgm.form import label, image
from pgm.layout import group
from pgm import widget

import sys, os
sys.path.insert(0,os.getcwd())

from elisa.core.widgets.action_item import ActionItem
from elisa.core.widgets import rectangle

class CheckBox(group.Group, ActionItem):

    def __init__(self, text, checked_img, unchecked_img,
                 checked_img_on, unchecked_img_on, background_img=None,
                 font_name='Sans', font_size=23, size=(50, 50),
                 on_action=None, checked=False):
        group.Group.__init__(self, size=size)

        self._bg = None
        self._label = None
        self.font_name = font_name
        self.font_size = font_size

        self.checked_img = image.Image(checked_img)
        self.unchecked_img = image.Image(unchecked_img)
        self.checked_img_on = image.Image(checked_img_on)
        self.unchecked_img_on = image.Image(unchecked_img_on)

        self.check_box_rect = rectangle.Rectangle(media=self.unchecked_img)

        self.background = background_img
        self.checked = checked
        self.text = text
        self.set_action_callback(self.toggle, on_action)

    def background__set(self, img_path):
        if img_path:
            if isinstance(img_path, image.Image):
                img = img_path
            else:
                img = image.Image(img_path)
            self._bg = rectangle.Rectangle()
            self._bg.ordering = widget.FAR
            self._bg.media = img
            self.add(self._bg)
        elif self._bg and self._bg in self:
            self.remove(self._bg)
            
    def text__set(self, txt):
        if self._label:
            self.remove(self._label)
        self._label = label.Label(size=self.size, text=txt,
                                  font_name=self.font_name,
                                  font_size=self.font_size)
        
        self._label.size = self._label.text_size
        self._label.width = self.width
        self.add(self._label)
        self._label.ordering = widget.NEAR
        self._label.x += self.check_box_rect.width
        self._label.yx_align = 1.
        
    def text__get(self):
        return self._label.text

    def checked__get(self):
        return self._checked

    def checked__set(self, value):
        self._checked = value
        self.opacity = self.opacity

    def img_check__set(self, img):
        self._check_img = img
        if self.check_box_rect not in self:
            self.add(self.check_box_rect)
        self.check_box_rect.media = img

    def img_check__get(self):
        return self._check_img

    def opacity__set(self, opacity):
        group.Group.opacity__set(self, opacity)
        if opacity == 255:
            if self.checked:
                img = self.checked_img_on
            else:
                img = self.unchecked_img_on
        else:
            if self.checked:
                img = self.checked_img
            else:
                img = self.unchecked_img
        self.img_check = img
        
    def toggle(self):
        self.checked = not self.checked

    def __repr__(self):
        return "<CheckBox checked=%r label=%r>" % (self.checked, self.text)

if __name__ == '__main__':

    from pgm import render
    from pgm.layout import window, group
    from pgm.form import rectangle, image
    import sys
    from pgm.timing import controller, modifier, keyframe
    from pgm import widget
    
    try:
        from twisted.internet import glib2reactor
        glib2reactor.install()
    except AssertionError:
        # already installed...
        pass
    
    from twisted.internet import reactor, task
    from twisted.spread import pb
    import gobject
    
    from elisa.core.widgets.list import VList

    pgmrender = render.init('opengl1')

    def key_pressed(event):
        if event.keyval == pgmrender.KEY_q:
            reactor.stop()
                
    def exit_event_cb(event):    
        win.main_quit()
        reactor.stop()
                    
    title = 'widget_level test'
    win = window.Window(title=title)

    b_size = (300, 50)
    checked_img = 'elisa/core/plugins/skins/default/pictures/unscaled/checked.png'
    unchecked_img = 'elisa/core/plugins/skins/default/pictures/unscaled/un_checked.png'
    checked_img_on = 'elisa/core/plugins/skins/default/pictures/unscaled/checked_on.png'
    unchecked_img_on = 'elisa/core/plugins/skins/default/pictures/unscaled/un_checked_on.png'
    box1 = CheckBox("foo", checked_img, unchecked_img,
                 checked_img_on, unchecked_img_on)
    box2 = CheckBox("bar", checked_img, unchecked_img,
                 checked_img_on, unchecked_img_on)
    box3 = CheckBox("bag", checked_img, unchecked_img,
                 checked_img_on, unchecked_img_on)
    
    li = VList((400, 200), spacing=5)
    li.add(box1)
    li.add(box2)
    li.add(box3)
    win.sig_key_press_event.connect(li.key_pressed)
    
    win.add(li)

    win.sig_key_press_event.connect(key_pressed)    
    
    _background_widget = rectangle.Rectangle((800,600))
    _background_widget.ordering = widget.FAR
    img = image.Image('elisa/core/plugins/skins/default/pictures/background.png')
    _background_widget.media = img
    _background_widget.tz = -1
    win.add(_background_widget)
    
    win.sig_exit_event.connect(exit_event_cb)
    win.sig_key_press_event.connect(key_pressed)
    gobject.timeout_add(15, win.update)

    win.main(reactor.loop)
    reactor.run()
