# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2007 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.


import os, sys

if __name__ == '__main__':
    sys.path.insert(0, os.getcwd())


from elisa.core.widgets.list import VList
from pgm.form import label
from pgm import widget
import textwrap

"""
TODO:

- scrollbar support
- key_press signal connection/deconnection

"""

class TextArea(VList):

    def __init__(self, size, spacing, text=None, columns=50,
                 font_name='Sans', font_size=23,
                 background_picture_path=None):
        VList.__init__(self, size, spacing, background_picture_path)
        self._labels = []
        self.columns = columns
        self.font_name = font_name
        self.font_size = font_size
        self.text = text

    def text__set(self, text):
        self._clean()
        self._text = text
        for line in textwrap.wrap(text, self.columns):
            labels = self._get_labels(line)
            for lbl in labels:
                lbl.y = len(self._labels) * self.spacing
                lbl.x_align = 0.
                lbl.y_align = 0.
                lbl.ordering = widget.NEAR
                lbl.tz = 1
                self.add(lbl)
                self._labels.append(lbl)

    def text__get(self):
        return self._text

    def _get_labels(self, line):
        labels = []
        size = (self.width, 30)
        lbl = label.Label(size=size, text=line,
                          font_name=self.font_name,
                          font_size=self.font_size)
        
        if lbl.text_size[0] == self.width:
            # split it up
            ok = False
            while not ok:
                words = line.split(' ')
                half = len(words)/2
                first_half = ' '.join(words[:half])
                second_half = ' '.join(words[half:])
                lbl1 = label.Label(size=size, text=first_half,
                                   font_name=self.font_name,
                                   font_size=self.font_size)
                lbl2 = label.Label(size=size, text=second_half,
                                   font_name=self.font_name,
                                   font_size=self.font_size)
                labels.extend([lbl1, lbl2])
                ok = lbl1.text_size[0] != self.width
                line = second_half
        else:
            labels = [lbl,]
        return labels

    def _clean(self):
        for lbl in self._labels:
            self.remove(lbl)
        self._labels = []


if __name__ == '__main__':

    from pgm import render
    from pgm.layout import window, group
    from pgm.form import rectangle, image
    import sys
    from pgm import widget
        
    try:
        from twisted.internet import glib2reactor
        glib2reactor.install()
    except AssertionError,e:
        # already installed...
        print e
        raise
        pass
    
    from twisted.internet import reactor, task
    from twisted.spread import pb
    import gobject
    
    pgmrender = render.init('opengl1')

    
    def key_pressed(event):
        if event.keyval == pgmrender.KEY_q:
            reactor.stop()
            
    def exit_event_cb(event):    
        win.main_quit()
        reactor.stop()
                    
    title = 'textarea test'
    win = window.Window(title=title,size=(800,600))

    text = "ceci est un long texte avec tout plein de mots et une seule phrase dedans, cette phrase est super beaucoup méga longue, et je me demande si l'accent que je viens de mettre là va passer... Mystère!   et boule de gomme quoi :)"
    area = TextArea((600,300),10, text, columns=40)
    win.add(area)
    
    _background_widget = rectangle.Rectangle((800,600))
    _background_widget.ordering = widget.FAR
    img = image.Image('elisa/core/plugins/skins/default/pictures/background.png')
    _background_widget.media = img
    _background_widget.tz = -1
    win.add(_background_widget)
    
    win.add(area)

    win.sig_exit_event.connect(exit_event_cb)
    win.sig_key_press_event.connect(key_pressed)
    gobject.timeout_add(15, win.update)

    win.main(reactor.loop)    
    reactor.run()
