# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

from twisted.internet import reactor
from elisa.core import common, log, media_uri
from elisa.core.signals.player import stopped_signal, playing_signal
import widget_item
from pgm.form import stream, image


class VideoWidgetItem(widget_item.WidgetItem):

    def __init__(self, menu_item, image=None, caption=None, start_size=(180,180),
                 arrow_image=None):
        self.menu_item = menu_item
        self.saved_bg = None
        self.dfr = None
        self.set_playing(False)
        widget_item.WidgetItem.__init__(self, background_image=image,
                                        caption=caption,
                                        start_size=start_size,
                                        arrow_image=arrow_image)
        stopped_signal.connect(self._player_stopped)
        playing_signal.connect(self._player_playing)
        self.saved_bg = None
        
    def get_menu_item(self):
        return self.menu_item

    def set_background_image(self, background_image):
        if not isinstance(background_image, stream.Stream):
            widget_item.WidgetItem.set_background_image(self,
                                                        background_image)
            if background_image:
                self.saved_bg = background_image
        else:
            self._main_widget.media = background_image

    def set_playing(self, value):
        self.playing = value

    def is_playing(self):
        return self.playing

    def _player_stopped(self, playable):
        uri = str(media_uri.Uri(self.menu_item.get_target_path()))
        if playable.get_uri() == uri and self.saved_bg:
            # FIXME: reset the skin's background before resetting our
            #        main_widget media because skin's background is the
            #        master and main_widget.media is the slave
            skin = common.get_application().get_skin()
            skin.reset_background()
            self._main_widget.media = self.saved_bg
            
    def _player_playing(self, playable, first_time):
        uri = str(media_uri.Uri(self.menu_item.get_target_path()))
        if playable.get_uri() == uri:
            player = common.get_application().get_player()
            self.set_background_image(player.get_stream())
            self.set_playing(True)
