# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

from pgm.form import rectangle, image, label, color, stroke
from pgm.timing import controller, modifier, keyframe
from pgm.layout import group
from pgm import widget
from elisa.core import common

class WidgetItem(group.Group):

    def __init__(self, background_image=None, caption='grr',
                 start_size=(180,180), arrow_image=None):
        self._main_widget = None
        self._arrow_widget = None
        self._label = None
        group.Group.__init__(self)
        self._item_original_width = start_size[0]
        self._main_widget = rectangle.Rectangle(start_size)
        self._main_widget.y_align = 1.0
        self.add(self._main_widget)
        self._caption = caption
        self.set_background_image(background_image)
        self._arrow_image = arrow_image
        self._arrow_visible = False
        self._label_visible = False
        self._is_current = False
        self._menu_item = None
        self._label = None
        
        # cartridge
        strok = stroke.Stroke(width=0,color=color.Color(color.BLACK))
        bg_color = color.Color((15,54,220,64))
        self._cartridge = rectangle.RoundRectangle(size=(800,20),
                                                   arc=(15,15),
                                                   position=(10,10),
                                                   stroke=strok,
                                                   paint=bg_color)
        self._cartridge.x_align = 1.0

        
    def set_menu_item(self, menu_item):
        self._menu_item = menu_item

    def get_menu_item(self):
        return self._menu_item
        
    def is_current(self):
        return self._is_current

    def set_current(self, value):
        self._is_current = value
##         menu_item = self.get_menu_item()
##         if menu_item:
##             if value:
##                 self._label.text = menu_item.get_short_name()
##                 if self._label not in self:
                    
##                     self.add(self._label)
##                     self._label.position = ((self._main_widget.width - self._label.width) /2, self._item_original_width)
##                     #self._label.position =
##             elif self._label in self:
##                 self.remove(self._label)
        
    def delete(self):
        self.remove(self._main_widget)

    def __repr__(self):
        return "<%s caption=%r bg=%r>" % (self.__class__.__name__, self._caption,
                                          self._background_image)

    def get_caption(self):
        return self._caption

    def set_caption(self, caption):
        self._caption = caption
        if self._label:
            if self._label_visible:
                self.remove(self._label)
                self.remove(self._cartridge)
            self._label = label.Label(size=(800, 30), text=self._caption,
                                      font_size=23)
            if self._label_visible:
                self.add(self._cartridge)
                self.add(self._label)
                if self.visible:
                    self._label.visible = True
                
            self._label.size = self._label.text_size
            if self._label.position != (0, 0):
                label_x = self.x + (self._main_widget.width - self._label.width) /2
                label_y = self.y + self._item_original_width

                self._label.position = (label_x, label_y)
                self._cartridge.ordering = widget.FAR
                self._cartridge.size = (self._label.width + 10,
                                        self._label.height)
                self._cartridge.position = (label_x - 5,
                                            label_y)

    def set_background_image(self, background_image):
        self._background_image = background_image
        if background_image != None:
            if isinstance(background_image, image.Image):
                img = background_image
            else:
                # FIXME: not sure of what to do in this case...
                img = image.Image(background_image)
            self._main_widget.media = img
        
    def get_background_image(self):
        return self._background_image
        
    def show_arrow(self):
        if self._arrow_image and not self._arrow_visible:
            label_height = 0
            if not self._arrow_widget:
                width = self._main_widget.width
                self._arrow_widget = rectangle.Rectangle((width,29))
                self._arrow_widget.added = False
                if isinstance(self._arrow_image, image.Image):
                    img = self._arrow_image
                else:
                    img = image.Image(self._arrow_image)
                self._arrow_widget.media = img
            label_height = self._label.height
            self._arrow_widget.position = (0,self._item_original_width + label_height + 4)
            self.add(self._arrow_widget)
            self._arrow_widget.visible = True
            self._arrow_visible = True
            
    def hide_arrow(self):
        # FIXME : change this by a hide when pigment will support this feature
        if self._arrow_visible:
            self.remove(self._arrow_widget)
            self._arrow_visible = False

    def show_label(self, dummy=False):
        if self._caption:
            if not self._label:
                self._label = label.Label(size=(800, 30), text=self._caption,
                                          font_size=23)
                self._label.x_align = 1.0
                #self._label.opacity = 255
                
            self._label.opacity = 255

            self._label.size = self._label.text_size
            #self._label.opacity = 0
            label_x = (self._main_widget.width - self._label.width) /2
            label_y = self._item_original_width
            
            if dummy:
                label_x += self.x
                label_y += self.y

            self._label.position = (label_x, label_y)
            self._cartridge.ordering = widget.FAR
            self._cartridge.size = (self._label.width + 10,
                                    self._label.height)
            self._cartridge.position = (self._label.x - 5,
                                        self._label.y)

            self.add(self._cartridge)
            self._cartridge.visible = True

            self.add(self._label)
            self._label.visible = True
            self._label_visible = True
            self._cartridge.opacity = 64

    def hide_label(self):
        if self._label_visible:
            self.remove(self._label)
            self.remove(self._cartridge)
            self._label_visible = False
        
    def size__set(self, size):
        if self._main_widget:
            self._main_widget.size__set(size)
        if self._arrow_widget:
            self._arrow_widget.width = size[0]
    
    def get_cartige_height(self):
        # 30 for the label
        # 10 is the space between image and police
        # 4 is space between police and arrow
        # 8 is arrow 
        return 30 + 10 + 4 + 8
    
    def size__get(self):
        if self._main_widget:
            return self._main_widget.size
        return None
 
    def size_mode__set(self, size_mode):
        if self._main_widget:
            self._main_widget.size_mode__set(size_mode)
            
    def size_mode__get(self):
        if self._main_widget:
            return self._main_widget.size_mode
        return None

    def visible__set(self, visible):
        group.Group.visible__set(self, visible)


    def opacity__set(self, opacity):
        group.Group.opacity__set(self, opacity)

        # FIXME: we always force cartridge opacity to 64.. widget_item opacity
        #        is updated in widget_level at various places.. 
        if self.visible and self._cartridge:
            self._cartridge.opacity = 64
            
                
    def rotate(self, uri, angle):
        img = self._main_widget.media
        #self.animated_rotate(img, angle)
        img.rotate(angle)
        self._main_widget.media = img
        

    def animated_rotate(self, img, angle):
        # TODO: angle in keyframe should be the new angle
        print 'angle=', angle
        duration = 3000
        timeline = [ keyframe.KeyFrame(0, 0),
                     keyframe.KeyFrame(angle, 1) ]

        def cb(ctr):
            img.rotate(angle)
            self._main_widget.media = img

        #from pgm import log
        #log.set_level('DEBUG')
        #print img, self._main_widget.media
        prop = 'rz'
        mod = modifier.Modifier([self, self._main_widget],
                                prop, timeline)
        self._ctrl = controller.Controller(begin=0, duration=duration,
                                           resolution=10,
                                           modifiers=[mod,],
                                           end_behavior=controller.END,
                                           interpolation=controller.SINUSOIDAL,
                                           end_callback=cb)

        appli = common.get_application()
        self._ctrl.start()
        

if __name__ == '__main__':  
    from pgm import render
    from pgm.layout import window, group
    from pgm.form import rectangle, image
    import sys
    from pgm.timing import controller, modifier, keyframe
    from pgm import widget
    
    try:
        from twisted.internet import glib2reactor
        glib2reactor.install()
    except AssertionError:
        # already installed...
        pass
    
    from twisted.internet import reactor, task
    from twisted.spread import pb
    import gobject
    import os
    
    def exit_event_cb(event):    
        win.main_quit()
        reactor.stop()
                    
    render.init('opengl1')
    title = 'widget_level test'
    win = window.Window(title=title)

    base = "elisa/core/plugins/skins/default"

    _background_widget = rectangle.Rectangle((800,600))
    _background_widget.ordering = widget.FAR
    img = image.Image(os.path.join(base,'pictures/background.png'))
    _background_widget.media = img
    _background_widget.tz = -1
    win.add(_background_widget)
    
    arrow_img = os.path.join(base,'pictures/128X128/music.png')
    
    #wi._main_widget.opacity = 0
    #wi._main_widget.paint.a = 0
    #wi.opacity = 0
    #import pdb; pdb.set_trace()

    g = group.Group()
    win.add(g)
    
    wi = WidgetItem(os.path.join(base,
                                 'pictures/128X128/music.png'), caption='bar',
                    arrow_image=arrow_img)
    wi2 = WidgetItem(os.path.join(base,
                                  'pictures/128X128/dvd.png'), caption=u'äöüßun',
                    arrow_image=arrow_img)

    wi.show_label()
    wi.show_arrow()
    g.add(wi)
    g.add(wi2)

    wi2.show_label()
    wi2.show_arrow()
    wi2.position=(200,0)
    #wi.opacity=100
    wi2.position=(200,0)

    wi.tz = 5
    wi.tz = 8
    wi.position = (300,100)
    """
    #wi.show_arrow()
    #wi.hide_arrow()
    #wi.x += 66
    print "wi=",wi.position
    print "main_w=",wi._main_widget.position
    print "arrow=",wi._arrow_widget.position
    #wi.hide_arrow()
    #wi.x += 11
    print "wi=",wi.position
    print "main_w=",wi._main_widget.position
    print "arrow=",wi._arrow_widget.position
    #wi.show_arrow()
    """
    

    win.sig_exit_event.connect(exit_event_cb)
    wi.tz = -150
    wi.y = 20
    gobject.timeout_add(15, win.update)

    win.main(reactor.loop)
    
    reactor.run()
    
